/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     | Website:  https://openfoam.org
    \\  /    A nd           | Copyright (C) 2020-2021 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#ifndef makeChemistrySolver_H
#define makeChemistrySolver_H

#include "addToRunTimeSelectionTable.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#define defineChemistrySolver(Model, ThermoPhysics)                            \
                                                                               \
    typedef Model<ThermoPhysics>                                               \
        Model##ThermoPhysics;                                                  \
                                                                               \
    defineTemplateTypeNameAndDebugWithName                                     \
    (                                                                          \
        Model##ThermoPhysics,                                                  \
        (                                                                      \
            word(Model##ThermoPhysics::typeName_())                            \
          + "<" + ThermoPhysics::typeName() + ">"                              \
        ).c_str(),                                                             \
        0                                                                      \
    )


#define makeChemistrySolver(Solver, Model, ThermoPhysics)                      \
                                                                               \
    typedef Solver<Model<ThermoPhysics>>                                       \
        Solver##Model##ThermoPhysics;                                          \
                                                                               \
    defineTemplateTypeNameAndDebugWithName                                     \
    (                                                                          \
        Solver##Model##ThermoPhysics,                                          \
        (                                                                      \
            word(Solver##Model##ThermoPhysics::typeName_())                    \
          + "<" + word(Model<ThermoPhysics>::typeName_())                      \
          + "<" + ThermoPhysics::typeName() + ">>"                             \
        ).c_str(),                                                             \
        0                                                                      \
    );                                                                         \
                                                                               \
    addToRunTimeSelectionTable                                                 \
    (                                                                          \
        basicChemistryModel,                                                   \
        Solver##Model##ThermoPhysics,                                          \
        thermo                                                                 \
    )


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#define defineChemistrySolvers(nullArg, ThermoPhysics)                         \
    defineChemistrySolver                                                      \
    (                                                                          \
        standardChemistryModel,                                                \
        ThermoPhysics                                                          \
    );                                                                         \
    defineChemistrySolver                                                      \
    (                                                                          \
        TDACChemistryModel,                                                    \
        ThermoPhysics                                                          \
    )

#define makeChemistrySolvers(Solver, ThermoPhysics)                            \
    makeChemistrySolver                                                        \
    (                                                                          \
        Solver,                                                                \
        standardChemistryModel,                                                \
        ThermoPhysics                                                          \
    );                                                                         \
    makeChemistrySolver                                                        \
    (                                                                          \
        Solver,                                                                \
        TDACChemistryModel,                                                    \
        ThermoPhysics                                                          \
    )


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
